@@ -474,6 +474,9 @@ STDTESTS = [
     'test_builtin',
     'test_exceptions',
     'test_types',
+    'test_unittest',
+    'test_doctest',
+    'test_doctest2',
    ]
 
 NOTTESTS = [
@@ -1246,6 +1249,9 @@ _expectations = {
         """,
     'openbsd3':
         """
+        test_audioop
+        test_rgbimg
+        test_imageop
         test_aepack
         test_al
         test_applesingle
